/*****************************************
  Between and within-subject effects
  Based on Rabe-Hesketh and Skrondal.

  Between-Subjects Variables:
     hsgrad somecoll collgrad married black

  Within-Subjects Variables:
     birwt smoke male mage
     kessner2 kessner3
     novisit pretri2 pretri3 
  
  filename=between_within.sas

*******************************************/
OPTIONS FORMCHAR="|----|+|---+=|-/\<>*";
options pageno=1 nodate;
title;

PROC IMPORT OUT= WORK.smoking 
            DATAFILE= "smoking.dta" 
            DBMS=STATA REPLACE;

RUN;

/*Number of births per mom*/
proc freq data=smoking noprint;
  tables momid/ out=tabdat;
run;
proc freq data=tabdat;
  tables count;
run;

/*Descriptives for Level 1 Variables*/
title "Level-1 (Within-Subjects) Variables";
proc means data=smoking;
  var smoke male mage 
      kessner2 kessner3 novisit 
      pretri2 pretri3;
run;

/*Generate Between-Subjects Data Set*/
proc sort data=smoking;
  by momid;
run;
title "Between-Subjects Variables";
data smoking_level2;
  set smoking;
  by momid;
  if first.momid;
  keep momid hsgrad somecoll collgrad married black;
run; 
proc means data=smoking_level2;
run;

/*Select a Random Sample of 50 Subjects*/
proc surveyselect data=smoking_level2 
     out=sampledat 
     method=srs 
     sampsize=50 
	 outall
     seed=442291000;
run;
data smoking_graph;
  merge smoking sampledat;
  by momid;
run;

/*Generate Boxplot*/
title "Boxplot Showing Between and Within Variability";
proc sgplot data=smoking_graph noautolegend;
   where selected=1;
   vbox birwt / category=momid;
   xaxis display=none;
run;


/*Linear Mixed Model with Random Intercept
  for each Subject*/
title "Linear Mixed Model";
title2 "Within-Subject Effects are not Strictly 'Within'";
proc mixed data=smoking;
  class momid;
  model birwt = smoke male mage kessner2 kessner3
                novisit pretri2 pretri3
                hsgrad somecoll collgrad
                married black  / solution;
  random int / subject=momid;
run;

/*Between-Subjects Effect Only model
  First aggregate within-subjects variables
  across subjects. Copy Within-Subjects Variables*/
proc sort data=smoking;
 by momid;
run;
proc means data=smoking noprint;
 by momid;
 output out=meandat
        mean(birwt)=mbirwt
        mean(smoke)=msmoke
		mean(male)=mmale
		mean(mage)=mmage
		mean(kessner2)=mkessner2
		mean(kessner3)=mkessner3
        mean(novisit)=mnovisit 
        mean(pretri2)=mpretri2
        mean(pretri3)=mpretri3;
 id hsgrad somecoll collgrad married black;
run;

title "Between-Subjects Effects Only";
proc reg data=meandat ;
  model mbirwt = msmoke mmale mmage 
                 mkessner2 mkessner3 
                 mnovisit mpretri2 mpretri3
                 hsgrad somecoll collgrad
                 married black ;
run;quit;


/*This takes a long time to run*/
title "Within-Subjects Analysis";
proc mixed data=smoking noclprint;
  class momid;
  model birwt =  smoke male mage kessner2 kessner3
                 novisit pretri2 pretri3 momid /solution;
run;

/*This runs very quickly*/
title "Alternate Within-Subjects Effects";
proc glm data=smoking ;
  model birwt = smoke male mage kessner2 kessner3 novisit 
                pretri2 pretri3 / solution ss3;
 absorb momid;
run; quit;

/*Setup for Between-Within Model Analysis*/
data smoking2;
  set  smoking;
       dbirwt=birwt;
       dsmoke=smoke;
       dmale=male;
       dmage=mage;
	   dkessner2=kessner2;
       dkessner3=kessner3;
       dnovisit=novisit;
       dpretri2=pretri2;
       dpretri3=pretri3;
run;

title "Create Deviations from Mom Mean";
title2 "For Within-Subjects Variables";
proc standard data=smoking2 mean=0 out=stddata;
  by momid;
  var  dbirwt
       dsmoke
       dmale
       dmage
	   dkessner2
       dkessner3
       dnovisit
       dpretri2
       dpretri3;
run;

/*Merge the Mean Data with the Deviations Data*/
data smoking3;
  merge stddata meandat;
  by momid;
run;

title "Hybrid Model: Between and Within Effects";
title2 "Plus Random Intercept Per Subject";
proc mixed data=smoking3 noclprint;
  class momid;
  model birwt = dsmoke dmale dmage dkessner2 dkessner3 dnovisit dpretri2 dpretri3
                msmoke mmale mmage mkessner2 mkessner3 mnovisit mpretri2 mpretri3
                hsgrad somecoll collgrad married black/ solution;
  random intercept / subject=momid; 
  contrast "smoking" dsmoke 1 msmoke -1;
  contrast "male" dmale 1 mmale -1; 
  contrast "mage" dmage 1 mmage -1;
  contrast "kessner" dkessner2 1 mkessner2 -1,
                     dkessner3 1 mkessner3 -1;
  contrast "pretri" dnovisit 1 mnovisit -1,
                    dpretri2 1 mpretri2 -1,
                    dpretri3 1 mpretri3 -1;
run;

proc datasets lib=work;
  delete re_data;
run;quit;

ods graphics on;

ods exclude solutionr;
title "Final BW Model";
title2 "Include Individual Effects Plus Means";
title3 "For variables that have different between and within effects"; 
proc mixed data=smoking3 noclprint plots=residualpanel plots(maxpoints=9000);
  class momid;
  model birwt = dsmoke 
                male
                dmage
                kessner2 kessner3
                novisit pretri2 pretri3 
        		msmoke mmage 
                hsgrad somecoll collgrad 
                married black			
				/ solution;
  random intercept / subject=momid solution; 
  ods output solutionr=re_data;
  run;

proc univariate data=re_data;
   var estimate;
   histogram/ normal(mu=est sigma=est);
   qqplot /normal(mu=est sigma=est);
run;



